#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "nr.h"
#include "mie_private.h"

#define Pi  3.14159

/*
  CALCULATE l* USING THE EXACT EQUATION 
  Original Program by P.D. Kaplan in Fortran, based on code by D. Pine and J. Xue with the Mie stuff but not the
     material parameters or comments.
  Converted to c by D. A. Boas
  Modified by X.D. Li 09/14/95
  It takes 
      index of refraction of Solvent "rindex", of microsphere "bindex",
      diamter of the microsphere "diam" (um),
      and volume fraction "ph" (absllute value, not the percentage) 
  as inputs

  18 Feb 97 - Calculate g as well  DAB
  18 Jan 2000 - remove minor bug in coef(), and redundant def'n of ph in main() and mie_poly() routines  P.D. Kaplan
*/

void mie_poly(double T, double diam, double Lambda, double *g, double *ls );
void mie(double rindex, double bindex,  double diam, double ph, double Lambda, double *g, double *ls );
double qromb(double (*func)(), double a, double b);



void main( int argc, char *argv[] )
{
	double T, diam, Lambda, g, ls;
	
	if( argc!=5 ) {
		printf( "USAGE: mie <T (K)> <diam (um)> <Vol. Frac.> <Lambda (nm)>\n" );
		exit(1);
	}

	T = atof( argv[1] );
	diam = atof( argv[2] );
	ph = atof( argv[3] );
	Lambda = atof( argv[4] );
	mie_poly( T, diam, Lambda, &g, &ls );

	printf( "\ng = %lf\n", g );
	printf( "ls = %lf\n", 1e-4*ls );
	printf( "l = %lf\n", 1e-4*ls*(1.-g) );
	printf( "musp = %lf\n", 1/ls/1e-4 );
	printf( "mus = %lf\n", 1/(1.-g)/ls/1e-4 );
}


void mie(double rindex, double bindex,  double diam, double ph, double Lambda, double *g, double *els )
{
/*  int i, j; */
  int nterms=100;
  double a, b, m;
  double k0, ap, bp, aveq, aveqs;
 
/* Get information about suspension
    rindex=index of solvent
    bindex=index of balls */

/* Lambda has units of nm*/
  m=bindex/rindex;
  Lambda=Lambda*1.0e-3;                  /* change Lambda to um*/
  k0=(2.0*Pi*rindex/Lambda);  
  yo=k0*diam;
  

  /* MIE COEFFICIENTS FOR THE SCATTERING AMPLITUDE */
  ap=.5*yo;
  bp=ap*m;
  coeff(&nterms,ap,bp,m);    /* nterm may be updated after the cal */
  nc = nterms;




  /* Calculate coefficients for S(q) */
  a1=-(1.0+2.0*ph)*(1.0+2.0*ph)/pow((1.0-ph),4.);
  a2=6.0*ph*pow((1.0+0.5*ph),2.)/pow((1.0-ph),4.);
  a4=0.5*ph*a1;
      
  /* CALCULATE l* */
      
  /* EXACT l* */
  a=zero;
  b=yo;
  aveqs = qromb(fsqq,a,b);
  aveq = aveqs*ph*48./(pow(yo,6.)*diam);   
  *els = 1./aveq;

  /* EXACT g */
  a=zero;
  b=yo;
  *g = 1. - 2.*qromb(fsqq,a,b)/(yo*yo*qromb(fs,a,b));


  printf( "musp = %e cm^{-1}       l* = %e cm\n", 1.e4/ *els, *els*1.e-4 );
  printf( "   g = %f\n", *g );
}


/*
  CALCULATE l* USING THE EXACT EQUATION 
  Orininal Program by P. Kaplan in Fortran.
  Converted to c by D. A. Boas
  Modified by X.D. Li 09/14/95
  It takes 
          temperature       T (K), 
          diamter of the microsphere diam (um),
          and volume fraction ph (absllute value, not the percentage) 
  as inputs

  18 Feb 97 - Calculate g as well  DAB
*/


void mie_poly(double T, double diam, double Lambda, double *g, double *els )
{
/*  int i, j; */
  int nterms=100;
  double rindex,bindex;
  double a, b, m;
  double k0, ap, bp, aveq, aveqs;

/* Get information about suspension
    rindex=index of solvent
    bindex=index of balls */

/* lambda has units of nm*/
  rindex = tlnh2o(T,Lambda);
  bindex = tlnps(T,Lambda);
  m=bindex/rindex;
  Lambda=Lambda*1.0e-3;                  /* change Lambda to um*/
  k0=(2.0*Pi*rindex/Lambda);  
  yo=k0*diam;

  /* MIE COEFFICIENTS FOR THE SCATTERING AMPLITUDE */
  ap=.5*yo;
  bp=ap*m;
  coeff(&nterms,ap,bp,m);    /* nterm may be updated after the cal */
  nc = nterms;


  /* Calculate coefficients for S(q) */
  a1=-(1.0+2.0*ph)*(1.0+2.0*ph)/pow((1.0-ph),4.);
  a2=6.0*ph*pow((1.0+0.5*ph),2.)/pow((1.0-ph),4.);
  a4=0.5*ph*a1;

  /* CALCULATE l* */
      
  /* EXACT l* */
  a=zero;
  b=yo;
  aveqs = qromb(fsqq,a,b);
  aveq = aveqs*ph*48./(pow(yo,6.)*diam);   
  *els = 1./aveq;

  /* EXACT g */
  a=zero;
  b=yo;
  *g = 1. - 2.*qromb(fsqq,a,b)/(yo*yo*qromb(fs,a,b));


  printf( "musp = %e cm^{-1}       l* = %e cm\n", 1.e4/ *els, *els*1.e-4 );
  printf( "   g = %f\n", *g );
  
}


/*
  ORIGINAL PROGRAM BY PETER KAPLAN

  THESE FUNCTIONS CONSTITUTE A DATABASE OF VALUES FOR THE
  INDEX OF REFRACTION OF WATER AND POLYSTYRENE AT VARIOUS
  TEMPERATURE AND WAVELENGTHS

  CONVERTED TO C BY DAVID BOAS
*/

double tlnh2o(double Tk, double lamnm)
{ 
  double lamum,Tc,temp;
  double n218,n120,c1,c2,c3,c4;
 
/*        these are fits to the data in Hale and Querry, Applied Optics
        v12, p555 (1973) Optical constants of water in the 200-nm to
        2000-um wavelength region.  They closely match (to better
        than .1%) the numbers
        used by Garcia-Rubio at Florida State who probably got his
        from the same source. */
  c1=-0.013414;
  c2=1.76148;
  c3=0.0065438;
  c4=-0.11512;
 
  lamum = 1.e-3*lamnm;
  Tc=Tk-273.;

  n218 = c1*lamum*lamum + c2 + c3/(lamum*lamum - c4*c4);
  n120 = sqrt(n218) + 1e-5 * (-15.11968);
 
  temp  = n120 - 1e-5 * (1.24e-1*(Tc-20.)+1.993e-1*(Tc*Tc - 400.)-5.0e-6*(Tc*Tc*Tc*Tc-160000.));

  return temp;
}



/*********************************************
         return index of polystyrene
*********************************************/
double tlnps(double Tk, double lamnm)
{
  double lamum,n,d1,d2,d3;

/*   This is fit to data
    from Devon&Rudin, File # 409 (J. Appl Polym. Sci  v34 p469 (1987))

  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        Now it's a fit to the data supplied by LH Garcia-Rubio
    of the University of South Florida's Chemical Engineering
    department.
        This fit produces results within .1% of Garcia-Rubio's data
    between 400 and 900 nm.  */
  d1=1.55165;
  d2=0.00766301;
  d3=0.000291013;

  lamum = lamnm * 1.e-3;
 
  n = d1 + d2/(lamum*lamum) + d3/(lamum*lamum*lamum*lamum);

  return n;
}



/* --------------------------------------------------------------------
      subroutine coeff(nc,alpha,beta,m)
c
c By D. Pine -- January 1988 / modified April 1988 / modified June 1989
c    some comments added Sept 1990 by Peter Kaplan
c
c This subroutine calcuates the Mie coefficients a(n) and b(n).  See
c
c     "Wave Propagation and Scattering in Random Media"
c     by Akira Ishimaru
c     Academic Press, New York (1978)
c     pp. 27-30
c and
c     "Light Scattering by Small Particles"
c     by H.C. van de Hulst
c     Dover Publications, New York 1957 & 1981
c     pp. 121-128, 176-177
c
c Glossary
c   alpha =  k*radius
c   beta =  k*m*radius
c   m = relative index of refraction
c   a(n) & b(n) = mie coefficients
c   nc = order up to which a(n) and b(n) are calculated
c
  Updates
    Converted to C by David Boas Jan 24 94
*/
void coeff(int *nterm, double alpha, double beta, double m)
{
  int n;
  dcomplex tp1, tp2, bt1 , bt2, mc;
  double psia[nmax], dpsia[nmax], psib[nmax], dpsib[nmax];
  double zetaa[nmax], dzetaa[nmax];
  double sna, csa, snb, csb;
  double fn, atest, btest, test;

  if( *nterm>nmax ) *nterm=nmax;
  printf("coef2 nterms=%i   ph = %lf\n",*nterm,ph);

/* ------ Initialize shorthand variables ----- */
  mc.r = m;
  mc.i = 0.;
  sna = sin(alpha);
  csa = cos(alpha);
  snb = sin(beta);
  csb = cos(beta);

/* Determine a(n) and b(n)
      The formula for a(n) & b(n) has spherical bessels and hankels.
      They are calculated here by explicitly writing out j1 and j2 then
      using recursion relations (see, for example Jackson, Classical
      Electrodynamics) to calculate the higher terms. --PDK    */
 

  for( n=1; n<*nterm; n++ )
    {
      fn = (double)n;
      if (n==1) 
	{
	  psia[1] = sna/alpha - csa;
	  dpsia[1] = sna - psia[1]/alpha;
	  zetaa[1] = -csa/alpha - sna;
	  dzetaa[1] = -csa - zetaa[1]/alpha;
	  psib[1] = snb/beta - csb;
	  dpsib[1] = snb - psib[1]/beta;
	}
      else if (n==2)
	{
	  psia[2] = (3.0/(alpha*alpha)-1.0)*sna - 3.0/alpha*csa;
	  dpsia[2] = psia[1] - 2.0/alpha*psia[2];
	  zetaa[2] = (-3.0/(alpha*alpha)+1.0)*csa - 3.0/alpha*sna;
	  dzetaa[2] = zetaa[1] - 2.0/alpha*zetaa[2];
	  psib[2] = (3.0/(beta*beta)-1.0)*snb - 3.0/beta*csb;
	  dpsib[2] = psib[1]-2.0/beta*psib[2];
	}
      else
	{
	  psia[n] = (2.0*fn-1.0)/alpha*psia[n-1] - psia[n-2];
	  dpsia[n] = psia[n-1] - fn/alpha*psia[n];
	  zetaa[n] = (2.0*fn-1.0)/alpha*zetaa[n-1] - zetaa[n-2];
	  dzetaa[n] = zetaa[n-1] - fn/alpha*zetaa[n];
	  psib[n] = (2.0*fn-1.0)/beta*psib[n-1] - psib[n-2];
	  dpsib[n] = psib[n-1] - fn/beta*psib[n];
	}

/*  this formula is in Ishimaru p29 */
      tp1 = dComplex(psia[n]*dpsib[n],0.0);
      tp2 = dComplex(psib[n]*dpsia[n],0.0);
      bt1 = dComplex(psia[n]*dpsib[n],zetaa[n]*dpsib[n]);
      bt2 = dComplex(psib[n]*dpsia[n],psib[n]*dzetaa[n]);
      as[n] = dCdiv( dCsub(tp1,dCmul(mc,tp2)), dCsub(bt1,dCmul(mc,bt2)) );
      bs[n] = dCdiv( dCsub(dCmul(mc,tp1),tp2), dCsub(dCmul(mc,bt1),bt2) );

/*  when the last calculated term is small, stop working */
      atest = sqrt( as[n].r*as[n].r + as[n].i*as[n].i );
      btest = sqrt( bs[n].r*bs[n].r + bs[n].i*bs[n].i );
      test = (2.0*fn+1.0)*(atest+btest);
      if( test<tol) break;
	 
    }

  *nterm = (int)fn;
  

}




/**********************************************************************
 Integrand for <q*q*F(q)*S(q)> --- same as fs(qa) but mul by xtra qa^2
**********************************************************************/
double fsqq(double qa)
{
  dcomplex f1, f2;
  double b, s;

  s=sq(qa);
 /* if (qa <=80) printf("fsqq qa=%lf  s=%lf\n",qa,s); */
  form(qa,&f1,&f2);
  b = f1.r*f1.r + f1.i*f1.i + f2.r*f2.r + f2.i*f2.i;

  return qa*qa*qa*b*s;
}
double fs(double qa)
{
  dcomplex f1, f2;
  double b, s;

  s=sq(qa);
  form(qa,&f1,&f2);
  b = f1.r*f1.r + f1.i*f1.i + f2.r*f2.r + f2.i*f2.i;
  return qa*b*s;
}



/**********************************************************************
 Calculates S(qa) (from Percus-Yevick result)
**********************************************************************/
double sq(double qa)
{
  double qqa, qqaa, sn, cs, sum1, sum2, sum4, cd;	

  qqa=2.0*qa;
  qqaa=qqa*qqa;
  sn=sin(qqa);
  cs=cos(qqa);
  sum1=sn - qqa*cs;
  sum2=2.0*qqa*sn - (qqaa-2.0)*cs-2.0;
  sum4=(qqaa*(12.-qqaa)-24.)*cs+24. + 4.0*qqa*(qqaa-6.0)*sn;
  cd=a1*sum1+(a2*sum2+a4*sum4/qqaa)/qqa;
  return 1.0/(1.0-3.0*ph*cd/(qa*qa*qa));
}



/**********************************************************************
 Calculate the scatteriing amplitude using mie theory.
 The amplitude recursion relations are derived by J. Xue
**********************************************************************/
void form(double y, dcomplex *f1, dcomplex *f2)
{
  int i, n;
  double x, fn, enx, eny;
  double sp[100],st[100];

/* FIRST TWO TERMS */
  x=1.-2.*y*y/(yo*yo);

/* STI(I) IS THE ITH LEGENDRE POLYNOMIAL,  STI(I) IS ITS DERIVATIVE */
  sp[1]=1.;
  st[1]=x;
  sp[2]=3.*x;
  st[2]=6.*x*x-3.;

/* THESE ARE RECURSION REL'NS TO CALCULATE HIGHER ORDER TERMS. */
  for( n=3; n<=nc; n++ )
    {
      fn=(double)n;
      sp[n] = ( (2.*fn-1.)*x*sp[n-1] - fn*sp[n-2] ) / (fn-1.);
      st[n] = fn*x*sp[n] - (fn+1.)*sp[n-1];
    }

/* F1 & F2 CORRESPOND TO S1 AND S2 IN ISHIMARU (S1 GOES WITH E PHI) */
  *f1=dComplex(0.,0.);
  *f2=dComplex(0.,0.);

/* CALCULATE THE ACTUAL SCATTERING AMPLITUDE. */
  for( i=1; i<=nc; i++ )
    {
      enx=(double)(2*i+1);
      eny=enx/(double)(i*i+i);
      f1->r = f1->r + eny*(as[i].r*sp[i] + bs[i].r*st[i]);
      f1->i = f1->i + eny*(as[i].i*sp[i] + bs[i].i*st[i]);
      f2->r = f2->r + eny*(bs[i].r*sp[i] + as[i].r*st[i]);
      f2->i = f2->i + eny*(bs[i].i*sp[i] + as[i].i*st[i]);
    }
}


